#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['koza',
 'koza.converter',
 'koza.io',
 'koza.io.reader',
 'koza.io.writer',
 'koza.manager',
 'koza.model',
 'koza.model.config']

package_data = \
{'': ['*']}

install_requires = \
['biolink_model_pydantic >=0.0.5,<0.1.0',
 'kgx >= 1.0.0',
 'pydantic >=1.0.0,<2.0.0',
 'pyyaml >=5.3.1,<6.0.0',
 'requests >=2.24.0,<3.0.0',
 'typer >=0.3']

extras_require = \
{'dev': ['autoflake >=1.3.1,<2.0.0',
         'flake8 >=3.8.3,<4.0.0',
         'black ==20.8b1',
         'isort >=5.0.6,<6.0.0'],
 'doc': ['mkdocs >=1.1.2,<2.0.0'],
 'test': ['pytest >=6.0.0']}

entry_points = \
{'console_scripts': ['koza = koza:main.app']}

setup(name='koza',
      version='0.0.1a5',
      description='Koza, an ETL framework for the Biolink model',
      author='The Monarch Initiative',
      author_email='info@monarchinitiative.org',
      url='https://github.com/monarch-initiative/koza',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )
